@@ -281,9 +281,17 @@ graphene_quaternion_slerp (const graphene_quaternion_t *a,
 {
   graphene_simd4f_t v_a = graphene_simd4f_init (a->x, a->y, a->z, a->w);
   graphene_simd4f_t v_b = graphene_simd4f_init (b->x, b->y, b->z, b->w);
+  float left_sign = 1;
 
   float dot = CLAMP (graphene_simd4f_get_x (graphene_simd4f_dot4 (v_a, v_b)), -1.f, 1.f);
 
+  /* Ensure we use the shortest path to the new angle */
+  if (dot < 0)
+    {
+      left_sign = -1;
+      dot = -dot;
+    }
+
   if (graphene_approx_val (dot, 1.f))
     {
       *res = *a;
@@ -299,7 +307,7 @@ graphene_quaternion_slerp (const graphene_quaternion_t *a,
   graphene_simd4f_t right = graphene_simd4f_init (b->x, b->y, b->z, b->w);
   graphene_simd4f_t sum;
 
-  left = graphene_simd4f_mul (left, graphene_simd4f_splat (left_v));
+  left = graphene_simd4f_mul (left, graphene_simd4f_splat (left_v * left_sign));
   right = graphene_simd4f_mul (right, graphene_simd4f_splat (right_v));
   sum = graphene_simd4f_add (left, right);
 
@@ -537,78 +545,7 @@ graphene_quaternion_t *
 graphene_quaternion_init_from_euler (graphene_quaternion_t  *q,
                                      const graphene_euler_t *e)
 {
-  graphene_euler_order_t order = graphene_euler_get_order (e);
-  const float ex = GRAPHENE_DEG_TO_RAD (graphene_euler_get_x (e)) / 2.f;
-  const float ey = GRAPHENE_DEG_TO_RAD (graphene_euler_get_y (e)) / 2.f;
-  const float ez = GRAPHENE_DEG_TO_RAD (graphene_euler_get_z (e)) / 2.f;
-  float c1, c2, c3, s1, s2, s3;
-  float s1c2c3, c1s2s3, c1s2c3, s1c2s3;
-  float c1c2s3, s1s2c3, c1c2c3, s1s2s3;
-
-  graphene_sincos (ex, &s1, &c1);
-  graphene_sincos (ey, &s2, &c2);
-  graphene_sincos (ez, &s3, &c3);
-
-  s1c2c3 = s1 * c2 * c3;
-  c1s2s3 = c1 * s2 * s3;
-  c1s2c3 = c1 * s2 * c3;
-  s1c2s3 = s1 * c2 * s3;
-  c1c2s3 = c1 * c2 * s3;
-  s1s2c3 = s1 * s2 * c3;
-  c1c2c3 = c1 * c2 * c3;
-  s1s2s3 = s1 * s2 * s3;
-
-  switch (order)
-    {
-    case GRAPHENE_EULER_ORDER_XYZ:
-      q->x = s1c2c3 + c1s2s3;
-      q->y = c1s2c3 - s1c2s3;
-      q->z = c1c2s3 + s1s2c3;
-      q->w = c1c2c3 - s1s2s3;
-      break;
-
-    case GRAPHENE_EULER_ORDER_YXZ:
-      q->x = s1c2c3 + c1s2s3;
-      q->y = c1s2c3 - s1c2s3;
-      q->z = c1c2s3 - s1s2c3;
-      q->w = c1c2c3 + s1s2s3;
-      break;
-
-    case GRAPHENE_EULER_ORDER_ZXY:
-      q->x = s1c2c3 - c1s2s3;
-      q->y = c1s2c3 + s1c2s3;
-      q->z = c1c2s3 + s1s2c3;
-      q->w = c1c2c3 - s1s2s3;
-      break;
-
-    case GRAPHENE_EULER_ORDER_ZYX:
-      q->x = s1c2c3 - c1s2s3;
-      q->y = c1s2c3 + s1c2s3;
-      q->z = c1c2s3 - s1s2c3;
-      q->w = c1c2c3 + s1s2s3;
-      break;
-
-    case GRAPHENE_EULER_ORDER_YZX:
-      q->x = s1c2c3 + c1s2s3;
-      q->y = c1s2c3 + s1c2s3;
-      q->z = c1c2s3 - s1s2c3;
-      q->w = c1c2c3 - s1s2s3;
-      break;
-
-    case GRAPHENE_EULER_ORDER_XZY:
-      q->x = s1c2c3 - c1s2s3;
-      q->y = c1s2c3 - s1c2s3;
-      q->z = c1c2s3 + s1s2c3;
-      q->w = c1c2c3 + s1s2s3;
-      break;
-
-    case GRAPHENE_EULER_ORDER_DEFAULT:
-      q->x = 0.f;
-      q->y = 0.f;
-      q->z = 0.f;
-      q->w = 1.f;
-      break;
-    }
+  graphene_euler_to_quaternion (e, q);
 
   return q;
 }
@@ -620,10 +557,21 @@ quaternion_equal (const void *p1,
   const graphene_quaternion_t *a = p1;
   const graphene_quaternion_t *b = p2;
 
-  return fabsf (a->x - b->x) < 0.00001 &&
-         fabsf (a->y - b->y) < 0.00001 &&
-         fabsf (a->z - b->z) < 0.00001 &&
-         fabsf (a->w - b->w) < 0.00001;
+  if (graphene_fuzzy_equals (a->x, b->x, 0.00001) &&
+      graphene_fuzzy_equals (a->y, b->y, 0.00001) &&
+      graphene_fuzzy_equals (a->z, b->z, 0.00001) &&
+      graphene_fuzzy_equals (a->w, b->w, 0.00001))
+    return true;
+
+  graphene_quaternion_t i;
+  graphene_quaternion_invert (a, &i);
+  if (graphene_fuzzy_equals (i.x, b->x, 0.00001) &&
+      graphene_fuzzy_equals (i.y, b->y, 0.00001) &&
+      graphene_fuzzy_equals (i.z, b->z, 0.00001) &&
+      graphene_fuzzy_equals (i.w, b->w, 0.00001))
+    return true;
+
+  return false;
 }
 
 /**
