@@ -133,6 +133,8 @@ GRAPHENE_AVAILABLE_IN_1_0
 float                   graphene_rect_get_width         (const graphene_rect_t *r);
 GRAPHENE_AVAILABLE_IN_1_0
 float                   graphene_rect_get_height        (const graphene_rect_t *r);
+GRAPHENE_AVAILABLE_IN_1_10
+float                   graphene_rect_get_area          (const graphene_rect_t *r);
 
 GRAPHENE_AVAILABLE_IN_1_0
 void                    graphene_rect_union             (const graphene_rect_t *a,
@@ -168,9 +170,12 @@ void                    graphene_rect_inset_r           (const graphene_rect_t
                                                          graphene_rect_t        *res);
 GRAPHENE_DEPRECATED_IN_1_4_FOR (graphene_rect_round)
 graphene_rect_t *       graphene_rect_round_to_pixel    (graphene_rect_t        *r);
-GRAPHENE_AVAILABLE_IN_1_4
+GRAPHENE_DEPRECATED_IN_1_10_FOR (graphene_rect_round_extents)
 void                    graphene_rect_round             (const graphene_rect_t  *r,
                                                          graphene_rect_t        *res);
+GRAPHENE_AVAILABLE_IN_1_10
+void                    graphene_rect_round_extents     (const graphene_rect_t  *r,
+                                                         graphene_rect_t        *res);
 GRAPHENE_AVAILABLE_IN_1_0
 void                    graphene_rect_interpolate       (const graphene_rect_t  *a,
                                                          const graphene_rect_t  *b,
