@@ -1,6 +1,6 @@
 /* graphene-simd4f.h: SIMD wrappers and operations
  *
- * Copyright © 2014  Emmanuele Bassi
+ * Copyright 2014  Emmanuele Bassi
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,7 @@
 /* needed for memcpy() */
 #include <string.h>
 #include <math.h>
+#include <float.h>
 
 #include "graphene-config.h"
 #include "graphene-macros.h"
@@ -763,7 +764,7 @@ _simd4f_neg (const graphene_simd4f_t s)
   return _mm_xor_ps (s, _mm_load_ps (__mask.f));
 }
 
-#else 
+#else
 
 #  error "Need GCC-compatible or Visual Studio compiler for SSE extensions."
 
@@ -1047,7 +1048,6 @@ typedef int graphene_simd4i_t __attribute__((vector_size (16)));
 #elif !defined(__GI_SCANNER__) && defined(GRAPHENE_USE_ARM_NEON)
 
 /* ARM Neon implementation of SIMD4f */
-# warning "The ARM Neon implementation of graphene_simd4f_t is experimental."
 
 /* Union type used for single lane reading without memcpy */
 typedef union {
@@ -1305,68 +1305,58 @@ typedef float32x2_t graphene_simd2f_t;
     (graphene_simd4f_t) vreinterpretq_f32_u32 (veorq_u32 (vreinterpretq_u32_f32 ((s)), __npnp)); \
   }))
 
-# define _graphene_movemask(a) \
-  (__extension__ ({ \
-    const int8_t __attribute__ ((aligned (16))) __xr[8] = { -7,-6,-5,-4,-3,-2,-1,0 }; \
-    const uint8x8_t __mask_and = vdup_n_u8 (0x80); \
-    const int8x8_t __mask_shift = vld1_s8 (__xr); \
-    uint8x8_t __lo = vget_low_u8 ((a)); \
-    uint8x8_t __hi = vget_high_u8 ((a)); \
-    __lo = vand_u8 (__lo, __mask_and); \
-    __lo = vshl_u8 (__lo, __mask_shift); \
-    __hi = vand_u8 (__hi, __mask_and); \
-    __hi = vshl_u8 (__hi, __mask_shift); \
-    __lo = vpadd_u8 (__lo, __lo); \
-    __lo = vpadd_u8 (__lo, __lo); \
-    __lo = vpadd_u8 (__lo, __lo); \
-    __hi = vpadd_u8 (__hi, __hi); \
-    __hi = vpadd_u8 (__hi, __hi); \
-    __hi = vpadd_u8 (__hi, __hi); \
-    (bool) ((__hi[0] << 8) | (__lo[0] & 0xff)); \
-  }))
-
 # define graphene_simd4f_cmp_eq(a,b) \
   (__extension__ ({ \
-    const graphene_simd4f_union_t __u_a = { (a) }; \
-    const graphene_simd4f_union_t __u_b = { (b) }; \
-    (bool) (__u_a.f[0] == __u_b.f[0] && \
-            __u_a.f[1] == __u_b.f[1] && \
-            __u_a.f[2] == __u_b.f[2] && \
-            __u_a.f[3] == __u_b.f[3]); \
+    const uint32x4_t __mask = vceqq_f32 ((a), (b)); \
+    (bool) (vgetq_lane_u32 (__mask, 0) != 0 && \
+            vgetq_lane_u32 (__mask, 1) != 0 && \
+            vgetq_lane_u32 (__mask, 2) != 0 && \
+            vgetq_lane_u32 (__mask, 3) != 0); \
   }))
 
 # define graphene_simd4f_cmp_neq(a,b) \
   (__extension__ ({ \
-    const graphene_simd4f_union_t __u_a = { (a) }; \
-    const graphene_simd4f_union_t __u_b = { (b) }; \
-    (bool) (__u_a.f[0] != __u_b.f[0] && \
-            __u_a.f[1] != __u_b.f[1] && \
-            __u_a.f[2] != __u_b.f[2] && \
-            __u_a.f[3] != __u_b.f[3]); \
+    const uint32x4_t __mask = vceqq_f32 ((a), (b)); \
+    (bool) (vgetq_lane_u32 (__mask, 0) == 0 || \
+            vgetq_lane_u32 (__mask, 1) == 0 || \
+            vgetq_lane_u32 (__mask, 2) == 0 || \
+            vgetq_lane_u32 (__mask, 3) == 0); \
   }))
 
 # define graphene_simd4f_cmp_lt(a,b) \
   (__extension__ ({ \
-    const uint8x16_t __mask = vreinterpretq_u8_u32 (vcltq_f32 ((a), (b))); \
-    (bool) (_graphene_movemask (__mask) != 0); \
+    const uint32x4_t __mask = vcltq_f32 ((a), (b)); \
+    (bool) (vgetq_lane_u32 (__mask, 0) != 0 && \
+            vgetq_lane_u32 (__mask, 1) != 0 && \
+            vgetq_lane_u32 (__mask, 2) != 0 && \
+            vgetq_lane_u32 (__mask, 3) != 0); \
   }))
 
 # define graphene_simd4f_cmp_le(a,b) \
   (__extension__ ({ \
-    const uint8x16_t __mask = vreinterpretq_u8_u32 (vcleq_f32 ((a), (b))); \
-    (bool) (_graphene_movemask (__mask) != 0); \
+    const uint32x4_t __mask = vcleq_f32 ((a), (b)); \
+    (bool) (vgetq_lane_u32 (__mask, 0) != 0 && \
+            vgetq_lane_u32 (__mask, 1) != 0 && \
+            vgetq_lane_u32 (__mask, 2) != 0 && \
+            vgetq_lane_u32 (__mask, 3) != 0); \
   }))
 
 # define graphene_simd4f_cmp_ge(a,b) \
   (__extension__ ({ \
-    const uint8x16_t __mask = vreinterpretq_u8_u32 (vcgeq_f32 ((a), (b))); \
-    (bool) (_graphene_movemask (__mask) != 0); \
+    const uint32x4_t __mask = vcgeq_f32 ((a), (b)); \
+    (bool) (vgetq_lane_u32 (__mask, 0) != 0 && \
+            vgetq_lane_u32 (__mask, 1) != 0 && \
+            vgetq_lane_u32 (__mask, 2) != 0 && \
+            vgetq_lane_u32 (__mask, 3) != 0); \
   }))
 
 # define graphene_simd4f_cmp_gt(a,b) \
   (__extension__ ({ \
-    const uint8x16_t __mask = vreinterpretq_u8_u32 (vcgeq_f32 ((a), (b))); \
-    (bool) (_graphene_movemask (__mask) != 0); \
+    const uint32x4_t __mask = vcgtq_f32 ((a), (b)); \
+    (bool) (vgetq_lane_u32 (__mask, 0) != 0 && \
+            vgetq_lane_u32 (__mask, 1) != 0 && \
+            vgetq_lane_u32 (__mask, 2) != 0 && \
+            vgetq_lane_u32 (__mask, 3) != 0); \
   }))
 
 # define graphene_simd4f_neg(s) \
@@ -1727,9 +1717,9 @@ graphene_simd4f_is_zero4 (const graphene_simd4f_t v)
 static inline bool
 graphene_simd4f_is_zero3 (const graphene_simd4f_t v)
 {
-  return graphene_simd4f_get_x (v) == 0.f &&
-         graphene_simd4f_get_y (v) == 0.f &&
-         graphene_simd4f_get_z (v) == 0.f;
+  return fabsf (graphene_simd4f_get_x (v)) <= FLT_EPSILON &&
+         fabsf (graphene_simd4f_get_y (v)) <= FLT_EPSILON &&
+         fabsf (graphene_simd4f_get_z (v)) <= FLT_EPSILON;
 }
 
 /**
@@ -1746,8 +1736,8 @@ graphene_simd4f_is_zero3 (const graphene_simd4f_t v)
 static inline bool
 graphene_simd4f_is_zero2 (const graphene_simd4f_t v)
 {
-  return graphene_simd4f_get_x (v) == 0.f &&
-         graphene_simd4f_get_y (v) == 0.f;
+  return fabsf (graphene_simd4f_get_x (v)) <= FLT_EPSILON &&
+         fabsf (graphene_simd4f_get_y (v)) <= FLT_EPSILON;
 }
 
 /**
