@@ -1,6 +1,6 @@
 /* graphene-simd4x4f.h: 4x4 float vector operations
  *
- * Copyright © 2014  Emmanuele Bassi
+ * Copyright 2014  Emmanuele Bassi
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,7 @@
 #include "graphene-simd4f.h"
 
 #include <math.h>
+#include <float.h>
 
 GRAPHENE_BEGIN_DECLS
 
@@ -67,8 +68,7 @@ GRAPHENE_BEGIN_DECLS
  *
  * Since: 1.0
  */
-GRAPHENE_VECTORCALL
-static inline graphene_simd4x4f_t
+static inline graphene_simd4x4f_t GRAPHENE_VECTORCALL
 graphene_simd4x4f_init (graphene_simd4f_t x,
                         graphene_simd4f_t y,
                         graphene_simd4f_t z,
@@ -218,8 +218,30 @@ graphene_simd4x4f_sum (const graphene_simd4x4f_t *a,
  * @b: a #graphene_simd4f_t
  * @res: (out): return location for a #graphene_simd4f_t
  *
- * Multiplies the given #graphene_simd4x4f_t with the given
- * #graphene_simd4f_t using a dot product.
+ * Left multiplies the given #graphene_simd4x4f_t with the given
+ * #graphene_simd4f_t row vector using a dot product:
+ *
+ * |[<!-- language="plain" -->
+ * res = b × A
+ *
+ *     = ⎡x⎤ ⎛ x.x  x.y  x.z  x.w ⎞
+ *       ⎜y⎟ ⎜ y.x  y.y  y.z  y.w ⎟
+ *       ⎜z⎟ ⎜ z.x  z.y  z.z  z.w ⎟
+ *       ⎣w⎦ ⎝ w.x  w.y  w.z  w.w ⎠
+ *
+ *     = [ x.x × x   x.y × x   x.z × x   x.w × x ]
+ *            +         +         +         +
+ *       [ y.x × y   y.y × y   y.z × y   y.w × y ]
+ *            +         +         +         +
+ *       [ z.x × z   z.y × z   z.z × z   z.w × z ]
+ *            +         +         +         +
+ *       [ w.x × w   w.y × w   w.z × w   w.w × w ]
+ *
+ *     = ⎡ x.x × x + y.x × y + z.x × z + w.x × w ⎤
+ *       ⎜ x.y × x + y.y × y + z.y × z + w.y × w ⎟
+ *       ⎜ x.z × x + y.z × y + z.z × z + w.z × w ⎟
+ *       ⎣ x.w × x + y.w × y + z.w × z + w.w × w ⎦
+ * ]|
  *
  * Since: 1.0
  */
@@ -246,9 +268,31 @@ graphene_simd4x4f_vec4_mul (const graphene_simd4x4f_t *a,
  * @v: a #graphene_simd4f_t
  * @res: (out): return location for a #graphene_simd4f_t
  *
- * Multiplies the given #graphene_simd4x4f_t with the given
+ * Left multiplies the given #graphene_simd4x4f_t with the given
  * #graphene_simd4f_t, using only the first three row vectors
- * of the matrix, and the first three components of the vector.
+ * of the matrix, and the first three components of the vector;
+ * the W components of the matrix and vector are ignored:
+ *
+ * |[<!-- language="plain" -->
+ * res = b × A
+ *
+ *     = ⎡x⎤ ⎛ x.x  x.y  x.z ⎞
+ *       ⎜y⎟ ⎜ y.x  y.y  y.z ⎟
+ *       ⎣z⎦ ⎝ z.x  z.y  z.z ⎠
+ *
+ *     = [ x.x × x   x.y × x   x.z × x ]
+ *            +         +         +
+ *       [ y.x × y   y.y × y   y.z × y ]
+ *            +         +         +
+ *       [ z.x × z   z.y × z   z.z × z ]
+ *
+ *     = ⎡ x.x × x + y.x × y + z.x × z ⎤
+ *       ⎜ x.y × x + y.y × y + z.y × z ⎟
+ *       ⎜ x.z × x + y.z × y + z.z × z ⎟
+ *       ⎣               0             ⎦
+ * ]|
+ *
+ * See also: graphene_simd4x4f_vec4_mul(), graphene_simd4x4f_point3_mul()
  *
  * Since: 1.0
  */
@@ -260,10 +304,12 @@ graphene_simd4x4f_vec3_mul (const graphene_simd4x4f_t *m,
   const graphene_simd4f_t v_x = graphene_simd4f_splat_x (*v);
   const graphene_simd4f_t v_y = graphene_simd4f_splat_y (*v);
   const graphene_simd4f_t v_z = graphene_simd4f_splat_z (*v);
+  graphene_simd4f_t r;
 
-  *res = graphene_simd4f_add (graphene_simd4f_add (graphene_simd4f_mul (m->x, v_x),
-                                                   graphene_simd4f_mul (m->y, v_y)),
-                              graphene_simd4f_mul (m->z, v_z));
+  r = graphene_simd4f_add (graphene_simd4f_add (graphene_simd4f_mul (m->x, v_x),
+                                                graphene_simd4f_mul (m->y, v_y)),
+                           graphene_simd4f_mul (m->z, v_z));
+  *res = graphene_simd4f_zero_w (r);
 }
 
 /**
@@ -276,7 +322,29 @@ graphene_simd4x4f_vec3_mul (const graphene_simd4x4f_t *m,
  * #graphene_simd4f_t.
  *
  * Unlike graphene_simd4x4f_vec3_mul(), this function will
- * also use the fourth row vector of the matrix.
+ * use the W components of the matrix:
+ *
+ * |[<!-- language="plain" -->
+ * res = b × A
+ *
+ *     = ⎡x⎤ ⎛ x.x  x.y  x.z  x.w ⎞
+ *       ⎜y⎟ ⎜ y.x  y.y  y.z  y.w ⎟
+ *       ⎜z⎟ ⎜ z.x  z.y  z.z  z.w ⎟
+ *       ⎣w⎦ ⎝ w.x  w.y  w.z  w.w ⎠
+ *
+ *     = [ x.x × x   x.y × x   x.z × x   x.w × x ]
+ *            +         +         +         +
+ *       [ y.x × y   y.y × y   y.z × y   y.w × y ]
+ *            +         +         +         +
+ *       [ z.x × z   z.y × z   z.z × z   z.w × z ]
+ *            +         +         +         +
+ *       [   w.x       w.y       w.z       w.w   ]
+ *
+ *     = ⎡ x.x × x + y.x × y + z.x × z + w.x ⎤
+ *       ⎜ x.y × x + y.y × y + z.y × z + w.y ⎟
+ *       ⎜ x.z × x + y.z × y + z.z × z + w.z ⎟
+ *       ⎣ x.w × x + y.w × y + z.w × z + w.w ⎦
+ * ]|
  *
  * Since: 1.0
  */
@@ -424,19 +492,18 @@ graphene_simd4x4f_matrix_mul (const graphene_simd4x4f_t *a,
    * better numbers while retaining the same correct results as above:
    * the scalar implementation now clocks at 91ns; the GCC vector
    * implementation is 19ns; and the SSE implementation is 16ns.
-   */
-  const graphene_simd4f_t row1 = a->x;
-  const graphene_simd4f_t row2 = a->y;
-  const graphene_simd4f_t row3 = a->z;
-  const graphene_simd4f_t row4 = a->w;
-
-  /* the order is correct if we want to multiply A with B; remember
+   *
+   * the order is correct if we want to multiply A with B; remember
    * that matrix multiplication is non-commutative.
    */
-  graphene_simd4x4f_vec4_mul (b, &row1, &res->x);
-  graphene_simd4x4f_vec4_mul (b, &row2, &res->y);
-  graphene_simd4x4f_vec4_mul (b, &row3, &res->z);
-  graphene_simd4x4f_vec4_mul (b, &row4, &res->w);
+  graphene_simd4f_t x, y, z, w;
+
+  graphene_simd4x4f_vec4_mul (b, &a->x, &x);
+  graphene_simd4x4f_vec4_mul (b, &a->y, &y);
+  graphene_simd4x4f_vec4_mul (b, &a->z, &z);
+  graphene_simd4x4f_vec4_mul (b, &a->w, &w);
+
+  *res = graphene_simd4x4f_init (x, y, z, w);
 #endif
 }
 
@@ -517,7 +584,7 @@ graphene_simd4x4f_init_ortho (graphene_simd4x4f_t *m,
  * graphene_simd4x4f_init_look_at:
  * @m: a #graphene_simd4x4f_t
  * @eye: vector for the camera coordinates
- * @center: vector the the object coordinates
+ * @center: vector for the object coordinates
  * @up: vector for the upwards direction
  *
  * Initializes a SIMD matrix with the projection necessary for
@@ -828,9 +895,11 @@ graphene_simd4x4f_div (const graphene_simd4x4f_t *a,
  *
  * Inverts the given #graphene_simd4x4f_t.
  *
+ * Returns: `true` if the matrix was invertible
+ *
  * Since: 1.0
  */
-static inline void
+static inline bool
 graphene_simd4x4f_inverse (const graphene_simd4x4f_t *m,
                            graphene_simd4x4f_t       *res)
 {
@@ -900,17 +969,24 @@ graphene_simd4x4f_inverse (const graphene_simd4x4f_t *m,
   const graphene_simd4f_t d0 = graphene_simd4f_mul (r1_sum, r0);
   const graphene_simd4f_t d1 = graphene_simd4f_add (d0, graphene_simd4f_merge_high (d0, d0));
   const graphene_simd4f_t det = graphene_simd4f_sub (d1, graphene_simd4f_splat_y (d1));
-  const graphene_simd4f_t invdet = graphene_simd4f_splat_x (graphene_simd4f_div (graphene_simd4f_splat (1.0f), det));
+  if (fabsf (graphene_simd4f_get_x (det)) >= FLT_EPSILON)
+    {
+      const graphene_simd4f_t invdet = graphene_simd4f_splat_x (graphene_simd4f_div (graphene_simd4f_splat (1.0f), det));
 
-  const graphene_simd4f_t o0 = graphene_simd4f_mul (graphene_simd4f_flip_sign_0101 (r1_sum), invdet);
-  const graphene_simd4f_t o1 = graphene_simd4f_mul (graphene_simd4f_flip_sign_1010 (r0_sum), invdet);
-  const graphene_simd4f_t o2 = graphene_simd4f_mul (graphene_simd4f_flip_sign_0101 (r3_sum), invdet);
-  const graphene_simd4f_t o3 = graphene_simd4f_mul (graphene_simd4f_flip_sign_1010 (r2_sum), invdet);
+      const graphene_simd4f_t o0 = graphene_simd4f_mul (graphene_simd4f_flip_sign_0101 (r1_sum), invdet);
+      const graphene_simd4f_t o1 = graphene_simd4f_mul (graphene_simd4f_flip_sign_1010 (r0_sum), invdet);
+      const graphene_simd4f_t o2 = graphene_simd4f_mul (graphene_simd4f_flip_sign_0101 (r3_sum), invdet);
+      const graphene_simd4f_t o3 = graphene_simd4f_mul (graphene_simd4f_flip_sign_1010 (r2_sum), invdet);
 
-  graphene_simd4x4f_t mt = graphene_simd4x4f_init (o0, o1, o2, o3);
+      graphene_simd4x4f_t mt = graphene_simd4x4f_init (o0, o1, o2, o3);
 
-  /* transpose the resulting matrix */
-  graphene_simd4x4f_transpose (&mt, res);
+      /* transpose the resulting matrix */
+      graphene_simd4x4f_transpose (&mt, res);
+
+      return true;
+    }
+
+  return false;
 }
 
 /**
@@ -1017,20 +1093,20 @@ graphene_simd4x4f_is_2d (const graphene_simd4x4f_t *m)
 {
   float f[4];
 
-  if (!(nearbyintf (graphene_simd4f_get_z (m->x)) == 0 && nearbyintf (graphene_simd4f_get_w (m->x)) == 0))
+  if (!(fabsf (graphene_simd4f_get_z (m->x)) < FLT_EPSILON && fabsf (graphene_simd4f_get_w (m->x)) < FLT_EPSILON))
     return false;
 
-  if (!(nearbyintf (graphene_simd4f_get_z (m->y)) == 0 && nearbyintf (graphene_simd4f_get_w (m->y)) == 0))
+  if (!(fabsf (graphene_simd4f_get_z (m->y)) < FLT_EPSILON && fabsf (graphene_simd4f_get_w (m->y)) < FLT_EPSILON))
     return false;
 
   graphene_simd4f_dup_4f (m->z, f);
-  if (!(nearbyintf (f[0]) == 0 &&
-        nearbyintf (f[1]) == 0 &&
-        nearbyintf (f[2]) == 1 &&
-        nearbyintf (f[3]) == 0))
+  if (!(fabsf (f[0]) < FLT_EPSILON &&
+        fabsf (f[1]) < FLT_EPSILON &&
+        1.f - fabsf (f[2]) < FLT_EPSILON &&
+        fabsf (f[3]) < FLT_EPSILON))
     return false;
 
-  if (!(nearbyintf (graphene_simd4f_get_z (m->w)) == 0 && nearbyintf (graphene_simd4f_get_w (m->w)) == 1))
+  if (!(fabsf (graphene_simd4f_get_z (m->w)) < FLT_EPSILON && 1.f - fabsf (graphene_simd4f_get_w (m->w)) < FLT_EPSILON))
     return false;
 
   return true;
