@@ -27,18 +27,13 @@
 
 #include <stdio.h>
 
-#if HAVE_PTHREAD
+#ifdef HAVE_PTHREAD
 #include <pthread.h>
 #include <errno.h>
 #include <string.h>
 #include <stdio.h>
 #endif
 
-#if HAVE_INIT_ONCE
-#define _WIN32_WINNT 0x0600
-#include <windows.h>
-#endif
-
 /**
  * SECTION:graphene-vectors
  * @Title: Vectors
@@ -330,7 +325,7 @@ void
 graphene_vec2_normalize (const graphene_vec2_t *v,
                          graphene_vec2_t       *res)
 {
-  if (graphene_vec2_length (v) != 0.f)
+  if (fabsf (graphene_vec2_length (v)) > FLT_EPSILON)
     res->value = graphene_simd4f_normalize2 (v->value);
   else
     res->value = graphene_simd4f_init_zero ();
@@ -408,6 +403,19 @@ graphene_vec2_negate (const graphene_vec2_t *v,
   res->value = graphene_simd4f_neg (v->value);
 }
 
+static bool
+vec2_equal (const void *p1,
+            const void *p2)
+{
+  const graphene_vec2_t *v1 = p1;
+  const graphene_vec2_t *v2 = p2;
+
+  if (graphene_simd4f_cmp_eq (v1->value, v2->value))
+    return true;
+
+  return graphene_vec2_near (v1, v2, GRAPHENE_FLOAT_EPSILON);
+}
+
 /**
  * graphene_vec2_equal:
  * @v1: a #graphene_vec2_t
@@ -423,16 +431,7 @@ bool
 graphene_vec2_equal (const graphene_vec2_t *v1,
                      const graphene_vec2_t *v2)
 {
-  if (v1 == v2)
-    return true;
-
-  if (v1 == NULL || v2 == NULL)
-    return false;
-
-  if (graphene_simd4f_cmp_eq (v1->value, v2->value))
-    return true;
-
-  return graphene_vec2_near (v1, v2, GRAPHENE_FLOAT_EPSILON);
+  return graphene_pointer_equal (v1, v2, vec2_equal);
 }
 
 /**
@@ -453,16 +452,8 @@ graphene_vec2_near (const graphene_vec2_t *v1,
                     const graphene_vec2_t *v2,
                     float                  epsilon)
 {
+  graphene_simd4f_t d = graphene_simd4f_sub (v1->value, v2->value);
   float epsilon_sq = epsilon * epsilon;
-  graphene_simd4f_t d;
-
-  if (v1 == v2)
-    return true;
-
-  if (v1 == NULL || v2 == NULL)
-    return false;
-
-  d = graphene_simd4f_sub (v1->value, v2->value);
 
   return graphene_simd4f_get_x (graphene_simd4f_dot2 (d, d)) < epsilon_sq;
 }
@@ -487,7 +478,7 @@ init_static_vec2_once (void)
   static_vec2[VEC2_Y_AXIS].value = graphene_simd4f_init (0.f, 1.f, 0.f, 0.f);
 }
 
-#if HAVE_PTHREAD
+#ifdef HAVE_PTHREAD
 static pthread_once_t static_vec2_once = PTHREAD_ONCE_INIT;
 
 static inline void
@@ -505,12 +496,13 @@ init_static_vec2 (void)
     }
 }
 
-#elif HAVE_INIT_ONCE
+#elif defined(HAVE_INIT_ONCE)
 static INIT_ONCE static_vec2_once = INIT_ONCE_STATIC_INIT;
 
-BOOL CALLBACK InitVec2Func (PINIT_ONCE InitOnce,
-                            PVOID Parameter,
-                            PVOID *lpContext)
+static BOOL CALLBACK
+InitVec2Func (PINIT_ONCE InitOnce,
+              PVOID      Parameter,
+              PVOID     *lpContext)
 {
   init_static_vec2_once ();
   return TRUE;
@@ -523,6 +515,7 @@ init_static_vec2 (void)
                                       InitVec2Func,
                                       NULL,
                                       NULL);
+
   if (!bStatus)
     fprintf (stderr, "InitOnceExecuteOnce failed\n");
 }
@@ -922,7 +915,7 @@ void
 graphene_vec3_normalize (const graphene_vec3_t *v,
                          graphene_vec3_t       *res)
 {
-  if (graphene_vec3_length (v) != 0.f)
+  if (fabsf (graphene_vec3_length (v)) > FLT_EPSILON)
     res->value = graphene_simd4f_normalize3 (v->value);
   else
     res->value = graphene_simd4f_init_zero ();
@@ -1089,6 +1082,19 @@ graphene_vec3_negate (const graphene_vec3_t *v,
   res->value = graphene_simd4f_neg (v->value);
 }
 
+static bool
+vec3_equal (const void *p1,
+            const void *p2)
+{
+  const graphene_vec3_t *v1 = p1;
+  const graphene_vec3_t *v2 = p2;
+
+  if (graphene_simd4f_cmp_eq (v1->value, v2->value))
+    return true;
+
+  return graphene_vec3_near (v1, v2, GRAPHENE_FLOAT_EPSILON);
+}
+
 /**
  * graphene_vec3_equal:
  * @v1: a #graphene_vec3_t
@@ -1104,16 +1110,7 @@ bool
 graphene_vec3_equal (const graphene_vec3_t *v1,
                      const graphene_vec3_t *v2)
 {
-  if (v1 == v2)
-    return true;
-
-  if (v1 == NULL || v2 == NULL)
-    return false;
-
-  if (graphene_simd4f_cmp_eq (v1->value, v2->value))
-    return true;
-
-  return graphene_vec3_near (v1, v2, GRAPHENE_FLOAT_EPSILON);
+  return graphene_pointer_equal (v1, v2, vec3_equal);
 }
 
 /**
@@ -1134,16 +1131,8 @@ graphene_vec3_near (const graphene_vec3_t *v1,
                     const graphene_vec3_t *v2,
                     float                  epsilon)
 {
+  graphene_simd4f_t d = graphene_simd4f_sub (v1->value, v2->value);
   float epsilon_sq = epsilon * epsilon;
-  graphene_simd4f_t d;
-
-  if (v1 == v2)
-    return true;
-
-  if (v1 == NULL || v2 == NULL)
-    return false;
-
-  d = graphene_simd4f_sub (v1->value, v2->value);
 
   return graphene_simd4f_dot3_scalar (d, d) < epsilon_sq;
 }
@@ -1170,7 +1159,7 @@ init_static_vec3_once (void)
   static_vec3[VEC3_Z_AXIS].value = graphene_simd4f_init (0.f, 0.f, 1.f, 0.f);
 }
 
-#if HAVE_PTHREAD
+#ifdef HAVE_PTHREAD
 static pthread_once_t static_vec3_once = PTHREAD_ONCE_INIT;
 
 static inline void
@@ -1188,12 +1177,13 @@ init_static_vec3 (void)
     }
 }
 
-#elif HAVE_INIT_ONCE
+#elif defined(HAVE_INIT_ONCE)
 static INIT_ONCE static_vec3_once = INIT_ONCE_STATIC_INIT;
 
-BOOL CALLBACK InitVec3Func (PINIT_ONCE InitOnce,
-                            PVOID Parameter,
-                            PVOID *lpContext)
+static BOOL CALLBACK
+InitVec3Func (PINIT_ONCE InitOnce,
+              PVOID      Parameter,
+              PVOID     *lpContext)
 {
   init_static_vec3_once ();
   return TRUE;
@@ -1206,6 +1196,7 @@ init_static_vec3 (void)
                                       InitVec3Func,
                                       NULL,
                                       NULL);
+
   if (!bStatus)
     fprintf (stderr, "InitOnceExecuteOnce failed\n");
 }
@@ -1676,7 +1667,7 @@ void
 graphene_vec4_normalize (const graphene_vec4_t *v,
                          graphene_vec4_t       *res)
 {
-  if (graphene_vec4_length (v) != 0.f)
+  if (fabsf (graphene_vec4_length (v)) > FLT_EPSILON)
     res->value = graphene_simd4f_normalize4 (v->value);
   else
     res->value = graphene_simd4f_init_zero ();
@@ -1788,6 +1779,19 @@ graphene_vec4_negate (const graphene_vec4_t *v,
   res->value = graphene_simd4f_neg (v->value);
 }
 
+static bool
+vec4_equal (const void *p1,
+            const void *p2)
+{
+  const graphene_vec4_t *v1 = p1;
+  const graphene_vec4_t *v2 = p2;
+
+  if (graphene_simd4f_cmp_eq (v1->value, v2->value))
+    return true;
+
+  return graphene_vec4_near (v1, v2, GRAPHENE_FLOAT_EPSILON);
+}
+
 /**
  * graphene_vec4_equal:
  * @v1: a #graphene_vec4_t
@@ -1803,16 +1807,7 @@ bool
 graphene_vec4_equal (const graphene_vec4_t *v1,
                      const graphene_vec4_t *v2)
 {
-  if (v1 == v2)
-    return true;
-
-  if (v1 == NULL || v2 == NULL)
-    return false;
-
-  if (graphene_simd4f_cmp_eq (v1->value, v2->value))
-    return true;
-
-  return graphene_vec4_near (v1, v2, GRAPHENE_FLOAT_EPSILON);
+  return graphene_pointer_equal (v1, v2, vec4_equal);
 }
 
 /**
@@ -1833,16 +1828,8 @@ graphene_vec4_near (const graphene_vec4_t *v1,
                     const graphene_vec4_t *v2,
                     float                  epsilon)
 {
+  graphene_simd4f_t d = graphene_simd4f_sub (v1->value, v2->value);
   float epsilon_sq = epsilon * epsilon;
-  graphene_simd4f_t d;
-
-  if (v1 == v2)
-    return true;
-
-  if (v1 == NULL || v2 == NULL)
-    return false;
-
-  d = graphene_simd4f_sub (v1->value, v2->value);
 
   return graphene_simd4f_get_x (graphene_simd4f_dot4 (d, d)) < epsilon_sq;
 }
@@ -1871,7 +1858,7 @@ init_static_vec4_once (void)
   static_vec4[VEC4_W_AXIS].value = graphene_simd4f_init (0.f, 0.f, 0.f, 1.f);
 }
 
-#if HAVE_PTHREAD
+#ifdef HAVE_PTHREAD
 static pthread_once_t static_vec4_init_once = PTHREAD_ONCE_INIT;
 
 static inline void
@@ -1889,12 +1876,13 @@ init_static_vec4 (void)
     }
 }
 
-#elif HAVE_INIT_ONCE
+#elif defined(HAVE_INIT_ONCE)
 static INIT_ONCE static_vec4_once = INIT_ONCE_STATIC_INIT;
 
-BOOL CALLBACK InitVec4Func (PINIT_ONCE InitOnce,
-                            PVOID Parameter,
-                            PVOID *lpContext)
+static BOOL CALLBACK
+InitVec4Func (PINIT_ONCE InitOnce,
+              PVOID      Parameter,
+              PVOID     *lpContext)
 {
   init_static_vec4_once ();
   return TRUE;
@@ -1907,6 +1895,7 @@ init_static_vec4 (void)
                                       InitVec4Func,
                                       NULL,
                                       NULL);
+
   if (!bStatus)
     fprintf (stderr, "InitOnceExecuteOnce failed\n");
 }
