@@ -111,6 +111,8 @@
 #define GRAPHENE_VERSION_1_2    (_GRAPHENE_ENCODE_VERSION (1, 2))
 #define GRAPHENE_VERSION_1_4    (_GRAPHENE_ENCODE_VERSION (1, 4))
 #define GRAPHENE_VERSION_1_6    (_GRAPHENE_ENCODE_VERSION (1, 6))
+#define GRAPHENE_VERSION_1_8    (_GRAPHENE_ENCODE_VERSION (1, 8))
+#define GRAPHENE_VERSION_1_10   (_GRAPHENE_ENCODE_VERSION (1, 10))
 
 #ifndef GRAPHENE_VERSION_MIN_REQUIRED
 # define GRAPHENE_VERSION_MIN_REQUIRED  (GRAPHENE_VERSION_1_0)
@@ -182,4 +184,34 @@
 # define GRAPHENE_AVAILABLE_IN_1_6              _GRAPHENE_PUBLIC
 #endif
 
+/* Graphene 1.8 */
+#if GRAPHENE_VERSION_MIN_REQUIRED >= GRAPHENE_VERSION_1_8
+# define GRAPHENE_DEPRECATED_IN_1_8             GRAPHENE_DEPRECATED
+# define GRAPHENE_DEPRECATED_IN_1_8_FOR(f)      GRAPHENE_DEPRECATED_FOR(f)
+#else
+# define GRAPHENE_DEPRECATED_IN_1_8             _GRAPHENE_PUBLIC
+# define GRAPHENE_DEPRECATED_IN_1_8_FOR(f)      _GRAPHENE_PUBLIC
+#endif
+
+#if GRAPHENE_VERSION_MAX_ALLOWED < GRAPHENE_VERSION_1_8
+# define GRAPHENE_AVAILABLE_IN_1_8              GRAPHENE_UNAVAILABLE(1,8)
+#else
+# define GRAPHENE_AVAILABLE_IN_1_8              _GRAPHENE_PUBLIC
+#endif
+
+/* Graphene 1.10 */
+#if GRAPHENE_VERSION_MIN_REQUIRED >= GRAPHENE_VERSION_1_10
+# define GRAPHENE_DEPRECATED_IN_1_10            GRAPHENE_DEPRECATED
+# define GRAPHENE_DEPRECATED_IN_1_10_FOR(f)     GRAPHENE_DEPRECATED_FOR(f)
+#else
+# define GRAPHENE_DEPRECATED_IN_1_10            _GRAPHENE_PUBLIC
+# define GRAPHENE_DEPRECATED_IN_1_10_FOR(f)     _GRAPHENE_PUBLIC
+#endif
+
+#if GRAPHENE_VERSION_MAX_ALLOWED < GRAPHENE_VERSION_1_10
+# define GRAPHENE_AVAILABLE_IN_1_10             GRAPHENE_UNAVAILABLE(1,10)
+#else
+# define GRAPHENE_AVAILABLE_IN_1_10             _GRAPHENE_PUBLIC
+#endif
+
 #endif /* __GRAPHENE_VERSION_MACROS_H__ */
